@@ -17,15 +17,15 @@ gMySQLBackend::gMySQLBackend(const string &mode, const string &suffix)  : GSQLBa
 {
   try {
     setDB(new SMySQL(getArg("dbname"),
-        	     getArg("host"),
-        	     getArgAsNum("port"),
-        	     getArg("socket"),
-        	     getArg("user"),
-        	     getArg("password"),
-        	     getArg("group")));
-    
+                     getArg("host"),
+                     getArgAsNum("port"),
+                     getArg("socket"),
+                     getArg("user"),
+                     getArg("password"),
+                     getArg("group"),
+                     mustDo("innodb-read-committed")));
   }
-  
+
   catch(SSqlException &e) {
     L<<Logger::Error<<mode<<" Connection failed: "<<e.txtReason()<<endl;
     throw AhuException("Unable to launch "+mode+" connection: "+e.txtReason());
@@ -48,6 +48,7 @@ public:
     declare(suffix,"password","Pdns backend password to connect with","");
     declare(suffix,"group", "Pdns backend MySQL 'group' to connect as", "client");
     declare(suffix,"dnssec","Assume DNSSEC Schema is in place","no");
+    declare(suffix,"innodb-read-committed","Use InnoDB READ-COMMITTED tranaction isolation level","yes");
 
     declare(suffix,"basic-query","Basic query","select content,ttl,prio,type,domain_id,name from records where type='%s' and name='%s'");
     declare(suffix,"id-query","Basic with ID query","select content,ttl,prio,type,domain_id,name from records where type='%s' and name='%s' and domain_id=%d");
