@@ -405,7 +405,6 @@ bool LUABackend::getDomainKeys(const string& name, unsigned int kind, std::vecto
 
     lua_pushnil(lua);  
 
-    int key;
     int j = 0;
     
     while (lua_next(lua, -2)) {
@@ -426,7 +425,6 @@ bool LUABackend::getDomainKeys(const string& name, unsigned int kind, std::vecto
         }
 
         lua_pop(lua,1);
-        key = lua_tonumber(lua, -1);
     }
 
     if(logging)
@@ -549,7 +547,6 @@ bool LUABackend::getDomainMetadata(const string& name, const std::string& kind,
 
     lua_pushnil(lua);  
 
-    int key;
     int j = 0;
     size_t returnedwhat;
     
@@ -561,7 +558,6 @@ bool LUABackend::getDomainMetadata(const string& name, const std::string& kind,
         }
 
         lua_pop(lua,1);
-        key = lua_tonumber(lua, -1);
     }
 
     if(logging)
@@ -593,7 +589,6 @@ void LUABackend::alsoNotifies(const string &domain, set<string> *ips) {
 
     lua_pushnil(lua);  
 
-    int key;
     size_t returnedwhat;
     
     while (lua_next(lua, -2)) {
@@ -603,7 +598,6 @@ void LUABackend::alsoNotifies(const string &domain, set<string> *ips) {
         }
 
         lua_pop(lua,1);
-        key = lua_tonumber(lua, -1);
     }
 
     if(logging)
