@@ -16,6 +16,7 @@
  * for even simple lookups is daft. It's quite trivial to craft a request
  * that'll require 128 database queries to answer with a servfail!
  *
+ * If you do not know what mydns is: http://mydns.bboy.net/
  */
 
 #include <string>
@@ -60,13 +61,14 @@ MyDNSBackend::MyDNSBackend(const string &suffix) {
         d_soatable=getArg("soa-table");
         d_rrwhere=(mustDo("rr-active")?"active = 1 and ":"")+getArg("rr-where");
         d_soawhere=(mustDo("soa-active")?"active = 1 and ":"")+getArg("soa-where");
+        d_useminimalttl=mustDo("use-minimal-ttl");
 
         L<<Logger::Warning<<backendName<<" Connection successful"<<endl;
 }
 
 MyDNSBackend::~MyDNSBackend() {
         if (d_db)
-        	delete(d_db);
+                delete(d_db);
 }
 
 
@@ -86,17 +88,14 @@ bool MyDNSBackend::list(const string &target, int zoneId) {
         d_db->setLog(::arg().mustDo("query-logging"));
 
         query = "select origin, minimum from "+d_soatable+" where id = ";
-        ostringstream o;
-        o<<zoneId;
-        query+=o.str();
-        query+= " and "+d_soawhere;
+        query+=itoa(zoneId);
+        if (!d_soawhere.empty())
+                query+= " and "+d_soawhere;
 
         this->Query(query);
 
-        if(!d_db->getRow(rrow)) {
-        	// No such zone
-        	return false;
-        }
+        if(!d_db->getRow(rrow))
+        	return false; // No such zone
         
         d_origin = rrow[0];
         if (d_origin[d_origin.length()-1] == '.')
@@ -104,12 +103,14 @@ bool MyDNSBackend::list(const string &target, int zoneId) {
         d_minimum = atol(rrow[1].c_str());
 
         while (d_db->getRow(rrow)) {
-        	L<<Logger::Warning<<backendName<<" Found more than one matching origin for zone ID: "+zoneId<<endl;
+        	L<<Logger::Warning<<backendName<<" Found more than one matching origin for zone ID: "<<zoneId<<endl;
         };
 
         query = "select type, data, aux, ttl, zone, name from "+d_rrtable+" where zone = ";
-        query+=o.str();
-        query += " and "+d_rrwhere;
+        query+=itoa(zoneId);
+        if (!d_rrwhere.empty())
+                query += " and "+d_rrwhere;
+        
 
         this->Query(query);
 
@@ -134,7 +135,9 @@ bool MyDNSBackend::getSOA(const string& name, SOAData& soadata, DNSPacket*) {
         else
         	query+=name;
 
-        query+=".' and "+d_soawhere;
+        query+=".'";
+        if (! d_soawhere.empty())
+                query += " and "+d_soawhere;
 
         this->Query(query);
 
@@ -151,7 +154,7 @@ bool MyDNSBackend::getSOA(const string& name, SOAData& soadata, DNSPacket*) {
         soadata.expire = atol(rrow[6].c_str());
         soadata.default_ttl = atol(rrow[7].c_str());
         soadata.ttl = atol(rrow[8].c_str());
-        if (soadata.ttl < soadata.default_ttl) {
+        if (d_useminimalttl && soadata.ttl < soadata.default_ttl) {
         	soadata.ttl = soadata.default_ttl;
         }
         soadata.db = this;
@@ -166,6 +169,7 @@ bool MyDNSBackend::getSOA(const string& name, SOAData& soadata, DNSPacket*) {
 void MyDNSBackend::lookup(const QType &qtype, const string &qname, DNSPacket *p, int zoneId) {
         string query;
         string sname;
+        string zoneIdStr = itoa(zoneId);
         SSql::row_t rrow;
         bool found = false;
 
@@ -193,11 +197,13 @@ void MyDNSBackend::lookup(const QType &qtype, const string &qname, DNSPacket *p,
         	pos = 0;
         	sdom = sname;
         	while (!sdom.empty() && pos != string::npos) {
-        		query = "select id, origin, minimum from "+d_soatable+" where origin = '"+sdom+"' and "+d_soawhere;
+        		query = "select id, origin, minimum from "+d_soatable+" where origin = '"+sdom+"'";
+                        if (!d_soawhere.empty()) 
+                                query += " and "+d_soawhere;
 
         		this->Query(query);
         		if(d_db->getRow(rrow)) {
-        			zoneId = atol(rrow[0].c_str());
+                                zoneIdStr=rrow[0];
         			d_origin = rrow[1];
         			if (d_origin[d_origin.length()-1] == '.')
         				d_origin.erase(d_origin.length()-1);
@@ -212,15 +218,14 @@ void MyDNSBackend::lookup(const QType &qtype, const string &qname, DNSPacket *p,
 
         } else {
         	query = "select origin, minimum from "+d_soatable+" where id = ";
-        	ostringstream o;
-        	o<<zoneId;
-        	query+=o.str();
-        	query+= " and "+d_soawhere;
+        	query+=zoneIdStr;
+                if (!d_soawhere.empty()) 
+        	       query+= " and "+d_soawhere;
 
         	this->Query(query);
 
         	if(!d_db->getRow(rrow)) {
-        		throw AhuException("lookup() passed zoneId = "+o.str()+" but no such zone!");
+        		throw AhuException("lookup() passed zoneId = "+zoneIdStr+" but no such zone!");
         	}
         	
         	found = true;
@@ -232,6 +237,7 @@ void MyDNSBackend::lookup(const QType &qtype, const string &qname, DNSPacket *p,
 
 
         if (found) {
+
         	while (d_db->getRow(rrow)) {
         		L<<Logger::Warning<<backendName<<" Found more than one matching zone for: "+d_origin<<endl;
         	};
@@ -248,17 +254,25 @@ void MyDNSBackend::lookup(const QType &qtype, const string &qname, DNSPacket *p,
         		host=d_db->escape(host);
 
         	query = "select type, data, aux, ttl, zone from "+d_rrtable+" where zone = ";
-        	ostringstream o;
-        	o<<zoneId;
-        	query+=o.str();
+        	query+= zoneIdStr;
         	query += " and (name = '"+host+"' or name = '"+sname+"')";
 
         	if(qtype.getCode()!=255) {  // ANY
         		query+=" and type='";
         		query+=qtype.getName();
         		query+="'";
+
         	}
-        	query += " and "+d_rrwhere+" order by type,aux,data";
+                if (!d_rrwhere.empty())
+                        query += " and "+d_rrwhere;
+
+
+                if (qtype.getCode() == 255) {
+                        query += " union select 'SOA' as type, origin as data, '0' as aux, ttl, id as zone from "+d_soatable+" where id= " + zoneIdStr + " and origin = '"+qname+".'";
+                        if (!d_soawhere.empty()) 
+                                query += " and " + d_soawhere;
+                }
+        	query += " order by type,aux,data";
 
         	this->Query(query);
 
@@ -294,6 +308,7 @@ bool MyDNSBackend::get(DNSResourceRecord &rr) {
         			rr.qname += ".";
         		rr.qname += d_origin; // Not fully qualified
         	}
+
         }
 
         if (rr.qtype.getCode() == QType::NS || rr.qtype.getCode()==QType::MX || 
@@ -309,13 +324,13 @@ bool MyDNSBackend::get(DNSResourceRecord &rr) {
 
         rr.priority = atol(rrow[2].c_str());
         rr.ttl = atol(rrow[3].c_str());
-        if (rr.ttl < d_minimum)
+        if (d_useminimalttl && rr.ttl < d_minimum)
         	rr.ttl = d_minimum;
         rr.domain_id=atol(rrow[4].c_str());
 
   
         rr.last_modified=0;
-  
+
         return true;
 
 }
@@ -338,6 +353,7 @@ public:
         	declare(suffix,"rr-where","Additional WHERE clause for RR","1 = 1");
         	declare(suffix,"soa-active","Use the active column in the SOA table","yes");
         	declare(suffix,"rr-active","Use the active column in the RR table","yes");
+        	declare(suffix,"use-minimal-ttl","Setting this to 'yes' will make the backend behave like MyDNS on the TTL values. Setting it to 'no' will make it ignore the minimal-ttl of the zone.","yes");
         }
 
         MyDNSBackend *make(const string &suffix = "") {
